
#
# $Copyright (c) 2018 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other 
# names may be trademarks of their respective owners.
# 
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $
#


package RPO;
use Date::Parse;

sub run_rc_cmd_ssh {
	my $cmd = shift;
	my $ssh_bin = shift;
	my $storage_server = shift;
	my $ssh_path_to_id_file = shift;
	my $user = shift;
	my  $rc_cmd = "";

	$rc_cmd =  "\"" .$ssh_bin."\" -l " . $user . " -i \"" . $ssh_path_to_id_file . "\" "
		. $storage_server . " \"" . $cmd. " \"". " 2>&1";


	my $out = `$rc_cmd`;

	$ret = $?;
	return ($ret, $out);
}

sub get_remotecopy_rpo_ssh {
	my $group = shift;
	my $storage_server = shift;
	my $ssh_bin = shift;
	my $ssh_path_to_id_file = shift;
	my $user = shift;
	my $sync_time_std = "";
	my $array_time_std = "";
	my $current_time = "";
	my $sync_time = 0;

	my $cmd = "showrcopy groups $group";
#showrcopy groups output
#Remote Copy System Information
#Status: Started, Normal
#
#Group Information
#
#Name                     Target     Status   Role       Mode     Options                  
#2577-rcg.r103096 RAAS3PAR01 Stopped  Secondary  Periodic Period 10m,over_per_alert
#  LocalVV              ID   RemoteVV     ID   SyncStatus    LastSyncTime           
#2577-hp3par 22611 2577 1698 Stale         2017-09-28 16:55:13 IST


	my ($ret, $out) =  run_rc_cmd_ssh($cmd, $ssh_bin, $storage_server, $ssh_path_to_id_file, $user);
	if ($ret != 0) {
		return (1, "Failed to get RemoteCopy group details. Cannot compute RPO");
	}

	my @out = split('\n', $out);
	while (@out) {
		$line = shift @out;
		if ($line =~ /Group Information/) {
			shift @out;
			my $status_line = shift @out;
			my $role_line = shift @out;
			my (@role_line) =  split(" ", $role_line);
			my $group_status = $role_line[2];
			my $mode = $role_line[4];
			if (($mode eq "Sync") && ($group_status eq "Started")) {
				return (0, "0");
			}
		}
		if ($line =~ /LocalVV/) {
			$found = 1;
			$line = shift @out;
			if ($line eq "") {
				return (1, "Virtual volumes are not configured for RemoteCopy group. Cannot compute RPO");
			}
		}
		if (($found == 1) && ($line ne "")) {
			my (@sync_line)  = split (" ", $line);
			$last_sync_time = join (" ", $sync_line[5],  $sync_line[6]);
			$sync_time_std =  $sync_line[7];
			my $temp_sync_time = str2time($last_sync_time);
			if (($temp_sync_time < $sync_time) || ($sync_time == 0)) {
				$sync_time = $temp_sync_time;
			}
		}
	}

#Get Current time of storage server.
#HPE3PAR01 cli% showdate
#Node Date                                  
#0    2018-10-02 22:30:34 IST (Asia/Kolkata)
#1    2018-10-02 22:30:52 IST (Asia/Kolkata)
#
	$cmd = "showdate";
	($ret, $out) =  run_rc_cmd_ssh($cmd, $ssh_bin, $storage_server, $ssh_path_to_id_file, $user);
	my @out = split('\n', $out);
	$line = shift @out;
	$line = shift @out;
	if($line =~ m/\d+\s+(.*?)\(/) {
		my (@sync_line)  = split (" ", $line);
		$current_time = join (" ", $sync_line[1],  $sync_line[2]);
		chomp $current_time;
		$array_time_std = $sync_line[3];
	}
	my $curr_time = str2time($current_time);
	if (($sync_time == 0) or ($current_time eq "")) {
		return (1, "Failed to get LastSyncTime or CurrentTime. Cannot compute RPO");
	}

	my $Rep_Tech_RPO = $curr_time - $sync_time;
	my $Replication_RPO= time_format($Rep_Tech_RPO);
	return (0, $Replication_RPO);
}


sub get_remotecopy_rpo_cli {
        my $group = shift;
        my $storage_server = shift;
        my $password_file = shift;
        my $cli = shift;
	my $ssh_path = shift;
	my $use_cli = shift;
	return 0;
}

sub time_format {
        my $time = shift;
        my ($hour, $min, $sec,$t1);
        $hour = "0";
        $min = "0";
        $sec = "0";
        if ($time >= 60){
                $t1 = int ($time / 60);
                $sec = ($time % 60);
                if ($t1 >= 60){
                        $hour = int ($t1/60);
                        $min = $t1 % 60;
                }else{
                        $min = ($t1);
                }
        }
        else {
                $sec = $time;
        }
        $hour = "0$hour" if ($hour < 10 && $hour >= 0);
        $min = "0$min" if ($min < 10 && $min >= 0);
        $sec = "0$sec" if ($sec < 10 && $sec >= 0);
        return "$hour:$min:$sec";
}




1;
